# -*- coding: utf-8 -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2015 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU Affero General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option)
#  any later version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
#  more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################

"""
Rattail Commands
"""

from __future__ import unicode_literals
from __future__ import absolute_import

import os
import re
import sys
import shutil
import zipfile

from rattail.commands import Subcommand


class AdminCommand(Subcommand):
    """
    Launches the 'SMS Admin' utility.  This tool is useful for administering
    SMS in both development and production environments.
    """

    name = 'sms-admin'
    description = "Launch 'SMS Admin' utility"

    def run(self, args):
        import wx
        from rattail_locsms.wx.Admin import AdminDialog

        app = wx.PySimpleApp()
        dlg = AdminDialog(None)
        dlg.ShowModal()
        dlg.Destroy()
        app.MainLoop()


class GenerateSampleCommand(Subcommand):
    """
    Generates a zipped Sample (or Option) file to an SMS install folder; called
    as ``rattail sms-gen-sample``.
    """

    name = 'sms-gen-sample'
    description = "Generate zipped SMS Sample (or Option) file"

    def add_parser_args(self, parser):
        parser.add_argument('source_dir',
                            help=r"Path to Sample source code (e.g. C:\Samples\MySample)")
        parser.add_argument('target_dir',
                            help=r"Path to Samples folder (e.g. C:\Storeman\install\Samples)")
        parser.add_argument('-t', '--fix-timestamp', action='store_true',
                            help="Set timestamp according to source control log")
        parser.add_argument('-m', '--mako-args', nargs='*', metavar='VAR=VAL',
                            help="Render *.mako files with specified Mako context vars")

    def run(self, args):
        from rattail_locsms.samples import generate_sample

        mako_arg_pattern = re.compile(r'^([A-Za-z_]+)=(.*)$')

        mako_args = {}
        if args.mako_args:
            for elem in args.mako_args:
                m = mako_arg_pattern.match(elem)
                if not m:
                    sys.stderr.write("Invalid Mako argument: %s\n" % elem)
                    return
                mako_args[m.group(1)] = eval(m.group(2))

        generate_sample(self.config, args.source_dir, args.target_dir,
                        render_mako=bool(mako_args),
                        use_source_time=args.fix_timestamp,
                        **mako_args)


class ImportSmsData(Subcommand):
    """
    Import data from a LOC SMS database.
    """
    name = 'import-locsms'
    description = "Import data from a LOC SMS database."

    def add_parser_args(self, parser):
        from rattail_locsms.importing import supported_model_names
        supported = supported_model_names()
        parser.add_argument('models', nargs='*', metavar='Model',
                            default=supported,
                            help="Name of model class(es) for which data should be imported.  "
                            "If not specified, data for all supported models will be imported.")
        parser.add_argument('--dry-run', action='store_true',
                            help="Go through the motions and allow logging to occur, "
                            "but do not actually commit the transaction at the end.")

    def run(self, args):
        from rattail_locsms.importing import make_provider
        from rattail.db import Session
        from rattail_locsms.db import configure_session_factory, Session as SmsSession
        from rattail_locsms.importing import supported_model_names

        supported = supported_model_names()
        session = Session()
        configure_session_factory(self.config)
        sms_session = SmsSession()
        for model_name in args.models:
            if model_name in supported:
                provider = make_provider(self.config, session, model_name, sms_session=sms_session)

                data = provider.get_data(progress=self.progress)
                sms_session.close()
                affected = provider.importer.import_data(data, provider.supported_fields, provider.key,
                                                         progress=self.progress)
                self.stdout.write("Added or updated {0} {1} records.\n".format(affected, provider.model_name))
            else:
                self.stderr.write("Model is not supported: {0}\n".format(repr(model_name)))

        if args.dry_run:
            session.rollback()
            self.stdout.write("Dry run, so transaction was rolled back.\n")
        else:
            session.commit()
        session.close()


class PullSampleCommand(Subcommand):
    """
    "Pulls" an SMS Sample (or Option) zip file into a source repository's
    working directory; called as ``rattail sms-pull-sample``.

    This tool will unzip the sample (or option) file and copy the individual
    files contained within it to the local working directory (which is assumed
    to be associated with a source repository).
    """

    name = 'sms-pull-sample'
    description = "Pull zipped SMS Sample (or Option) file"

    def add_parser_args(self, parser):
        parser.add_argument('sample_file',
                            help=r"Path to zipped Sample (e.g. C:\Storeman\install\Samples\MySample.zip)")
        parser.add_argument('source_dir',
                            help=r"Path to source working directory (e.g. C:\Samples\MySample)")

    def run(self, args):
        from rattail_locsms.samples import absorb_sample
        absorb_sample(args.sample_file, args.source_dir)
